/*
 * Created on Jul 8, 2004
 *
 */
package com.qanavigation.XMLDBTest;

import java.util.ArrayList;

import com.qanavigation.Basics.IRslt;
import com.qanavigation.Basics.Rslt;
import com.qanavigation.Basics.Rslt2DArray;
import com.qanavigation.Basics.RsltStatVal;
import com.qanavigation.Basics.RsltString;
import com.qanavigation.Basics.RsltTable;
import com.qanavigation.DBacc.DBacc2;
import com.qanavigation.XMLBasics.RsltXMLDoc;
import com.qanavigation.XMLBasics.RsltXMLNode;
import com.qanavigation.XMLBasics.RsltXMLNodeLst;
import com.qanavigation.XMLDB.XMLDB;
import com.qanavigation.XMLDB.XMLDBNode;

/**
 * @author RDG
 *
 */
public class XMLDBTest {
	
	private DBacc2 m_dbAcc;
	private XMLDB m_xDB;

	/**
	 * 
	 * @param args: DBName, DB location, DB user, DB user password
	 */
	public static void main( String[] args ) {
	
		XMLDBTest tst = new XMLDBTest( "QNdata", "localhost", "QANuser", "QANPass");
		
		tst.run();
	}
	
	/**
	 * Constructor. Creates a new DB access object.
	 * @param p_DBName
	 * @param p_Location
	 * @param p_User
	 * @param p_PW
	 */
	public XMLDBTest( String p_DBName, String p_Location, String p_User, String p_PW ) {
		
		m_dbAcc = new DBacc2( p_DBName, p_Location, p_User, p_PW );
	}
	
	/**
	 * Runs all test and prints out errors, if occured.
	 */
	public void run() {
		
		Rslt res = new Rslt( true );
		Rslt res2;
		RsltString rUID, rStr;
		String nodeName, attrName, attrVal;
		
		try {

			if( res.checkRslt( m_dbAcc ) ) {

				System.out.println( "Creating new XMLDB object" );
				System.out.println( "Requires the UID server! Make sure it is running! " );
				
				m_xDB = new XMLDB( m_dbAcc, "localhost", 6194 );

				if( res.checkRslt( m_xDB ) ) {
					
					nodeName = "TestNode";
					attrName = "TestAttr";
					attrVal = "TestVal";
					
					System.out.println( "Create a new XML and store it." );
					rUID = test_CreatNewDBXML( nodeName, attrName, attrVal );
					
					if( res.checkRslt( rUID ) ) {
						
						System.out.println( "New XML with UID '" + rUID.getString() + "' created." );
						
						res = test_Srch( nodeName, attrName, attrVal );
					}
					
					if( res.isOK() ) {
						
						System.out.println( "Trying to append a node." );
						rStr = test_AppendNode( rUID );
						
						if( res.checkRslt( rStr ) ) {
							
							System.out.println(  "Appended node UID is '" + rStr.getString() + "'." );
						}
					}
					
					if( rUID.isOK() ) {
						
						System.out.println( "Delete the whole XML." );
						res2 = test_DeleteDBXML( rUID );
						
						res.checkRslt( res2 );
					}
				}
				
				m_dbAcc.close();				
			}
		}
		catch( Exception excp ) {
			
			res.setRsltByExcp( excp );
		}
		
		issueIfErrors( res );
	
		System.out.println( "Done." );
		
	}
	
	/**
	 * Creates a new XML and stores it.
	 * @return The UID.
	 */
	private RsltString test_CreatNewDBXML( String p_nodeName, String p_attrName, String p_attrVal ) {
		
		RsltString rUID;
		RsltXMLDoc xDoc;
		RsltXMLNode tNode;

		rUID = new RsltString( true );

		xDoc = new RsltXMLDoc( "", p_nodeName, null, null );

		if( rUID.checkRslt( xDoc ) ) {

			tNode = xDoc.getTopNode();

			if( rUID.checkRslt( tNode ) ) {

				tNode.addAttribute( p_attrName, p_attrVal );
			}

			tNode.appTxtNode( "TestNode", "This is a test!" );

			rUID = m_xDB.importXML( xDoc.m_xDoc );
		}

		return rUID;
	}
	
	/**
	 * 
	 * @param p_UID
	 * @return The result
	 */
	private Rslt test_DeleteDBXML( RsltString p_UID ) {
		
		Rslt res;

		res = m_xDB.deleteXML( p_UID.getString() );
		
		return res;
	}
	
	// -------------------------------------------
	private Rslt test_Srch( String p_nodeName, String p_attrName, String p_attrVal ) {
		
		Rslt res = new Rslt( true );
		
		Rslt2DArray attrArr;
		RsltTable rTbl;
		
		String[] row;
		
		attrArr = new Rslt2DArray( 2 );
		
		row = new String[2];
		
		row[0] = p_attrName;
		row[1] = p_attrVal;
		
		attrArr.add( row );
		
		System.out.println( "Searching Nodes '" + p_nodeName + "' containing attributes '" + p_attrName + "' with '" + p_attrVal + "'." );

		rTbl = m_xDB.getNodeLstByAttrs( p_nodeName, attrArr );
		
		if( res.checkRslt( rTbl ) ) {
		
			System.out.println( rTbl.size() );
		}
		
		return res;
	}

	// -------------------------------------------
	private RsltString test_AppendNode( RsltString p_uid ) {

		RsltString res = new RsltString( true );
		
		RsltXMLNode xNode;
		RsltTable rTbl;
		
		String[] row;
		
		if( res.checkRslt( p_uid ) ) {
			
			rTbl = m_xDB.getXNodeLstByParent( p_uid.getString() );
			
			if( res.checkRslt( rTbl ) ) {
				
				if( rTbl.getRowNum() > 0 ) {
					
					row = rTbl.get( 0 );
					
					System.out.println( "Node '" + row[0] + "', named '" + row[1]+ "', type '" + row[2] + "' found." );
					
					xNode = new RsltXMLNode( "AppNode" );
					
					xNode.addAttribute( "TestAttr", "Test" );
					xNode.appTxt( "Some Text." );
		
					res = m_xDB.appSubTree( row[0], xNode, false );
				}
			}
		}

		return res;
	}
		
	// -------------------------------------------
	static void issueIfErrors( IRslt res ) {

		int i, x;
		ArrayList<String> addInfo;
		
		if( res.notOK() ) {

			addInfo = res.getRsltAddInfo();
	
			System.out.println( "Messages:" );
	
			x = addInfo.size();
	
			for( i = 0; i < x; ++i ) {
	
				System.out.println( addInfo.get( i ) );
			}
		
			System.out.println( "Status: " + RsltStatVal.label[ res.getRsltStatVal() ] );
		}
	}
}
